#include <Silence.hpp>
#include "csound.hpp" 
#include "CsoundFile.hpp" 

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <fstream>
#include <cstdlib>
#include <string.h>
#include <sstream>
#include <iostream>

#include "xml.h"
#include "xmlfile.h"
#include "xmlreader.h"
#include "midicontextvisitor.h"
#include "unrolled_xml_tree_browser.h"
#include "xml_tree_browser.h"


class myMidiWriter : public midiwriter
{
public:
    long tpq;
    double tempo;
    double myinstNum = 0;
    myMidiWriter(Score *score_) : tpq(480), tempo(1.0), score(*score_)
    {
        score.clear();
        score.removeArrangement();
    }
 
    virtual void startPart (int instrCount)
    {
    }
    virtual void newInstrument (std::string instrName, int chan=-1)
    {
        myinstNum++;
    }
    virtual void endPart (long date)
    {
    }
    virtual void newNote (long start_, int channel_, float key_, int velocity_, int duration_)
    {
        double start = double(start_) / double(tpq) * tempo;
        double duration = double(duration_) / double(tpq) * tempo;
        double status = 144.0;
        double channel = myinstNum;
        double key = key_;
        double velocity = velocity_;
        score.append(start, duration, status, channel, key, velocity);
    }
    virtual void tempoChange (long date, int bpm)
    {
        tempo = 60.0 / double(bpm);
    }
    virtual void pedalChange (long date, pedalType t, int value)
    {
    }
    virtual void volChange (long date, int chan, int vol)
    {
    }
    virtual void bankChange (long date, int chan, int bank)
    {
    }
    virtual void progChange (long date, int chan, int prog)
    {
    }
public:
    Score &score;
};
 




class myscore : public Score
{
public:
    virtual void myload(std::string filename);
};
 
void myscore::myload(std::string filename)
{
 
    System::inform("BEGAN myScore::load(%s)...\n", filename.c_str());
    if (filename.find(".xml") != std::string::npos ||
            filename.find(".XML") != std::string::npos)
    {
        xmlreader xmlReader;
        Sxmlelement sxmlElement;
        SXMLFile sxmlFile = xmlReader.read(filename.c_str());
        if (sxmlFile)
        {
            sxmlElement = sxmlFile->elements();
        }
        if (sxmlElement)
        {
            myMidiWriter scoreMidiWriter(this);
            midicontextvisitor midicontextvisitor_(scoreMidiWriter.tpq, &scoreMidiWriter);
            xml_tree_browser xmlTreeBrowser(&midicontextvisitor_);
            xmlTreeBrowser.browse(*sxmlElement);
        }
    }
 
    else
    {
        System::error("Unknown file format in myScore::load().\n");
    }
    System::inform("ENDED myScore::load().\n");
}




int main()
{
    csound::MusicModel model;
    CppSound *csound = model.getCppSound(); //sets the orchestra
    csound::Score &score = model.getScore();
    myscore ms;
 
    ms.myload("/path to your file/myxmlfile.xml");
 
    ms.sort();
    cout << ms.getCsoundScore() << endl; 
    model.createCsoundScore(ms.getCsoundScore()); 
    csound->getScore();
    csound->save("/path to save your file/myfile.sco");
    return 0;
}
 
